
type t = Set of Ints.t | Coset of Ints.t

let equal d1 d2 = match d1, d2 with
| Set s1, Set s2 -> Ints.equal s1 s2
| Coset s1, Coset s2 ->  Ints.equal s1 s2
| _,_ -> false

let empty = Set Ints.empty
and universe = Coset Ints.empty

open Printf

let pints chan is =
  let rec do_rec = function
    | [] -> ()
    | [x] -> fprintf chan "%d" x
    | x::xs -> fprintf chan "%d, " x ; do_rec xs in
  do_rec (Ints.elements is)

let pretty chan = function
  | Coset s when Ints.is_empty s -> fprintf chan "*"
  | Coset s -> fprintf chan "~{%a}" pints s
  | Set s -> fprintf chan "{%a}" pints s

let singleton i = Set (Ints.singleton i)
and cosingleton i = Coset (Ints.singleton i)

let of_ints l =
  Set (List.fold_left (fun r i -> Ints.add i r) Ints.empty l)

let is_empty = function
  | Set s   -> Ints.is_empty s
  | Coset _ -> false

let inter d1 d2 = match d1, d2 with
| Set s1, Set s2   -> Set (Ints.inter s1 s2)
| Set s1, Coset s2 -> Set (Ints.diff s1 s2)
| Coset s1, Set s2 -> Set (Ints.diff s2 s1)
| Coset s1, Coset s2 -> Coset (Ints.union s1 s2)

let complement = function
  | Set is -> Coset is
  | Coset is -> Set is


let inter_elem d x = match d with
| Set s   -> if Ints.mem x s then singleton x else empty
| Coset s -> if Ints.mem x s then empty else singleton x

let inter_coelem d x = match d with
| Set s   -> Set (Ints.remove x s)
| Coset s -> Coset (Ints.add x s)

type decide = Oui | Non | SaisPas

let decide x d = match d with
| Set s ->
    if Ints.mem x s  then
      if Ints.is_empty (Ints.remove x s) then
        Oui
      else
        SaisPas
    else
      Non
| Coset s ->
    if Ints.mem x s then
      Non
    else
      SaisPas

