module Ord =
struct
  type t=int

  let compare x y = x - y
end
  
module IntSet = Set.Make (Ord)

open IntSet

type elt = int
type t = IntSet.t
let empty = empty
          (* The empty set. *)
let is_empty = is_empty
        (* Test whether a set is empty or not. *)
let mem = mem
        (* [mem x s] tests whether [x] belongs to the set [s]. *)
let add = add
        (* [add x s] returns a set containing all elements of [s],
           plus [x]. If [x] was already in [s], [s] is returned unchanged. *)
let singleton = singleton
        (* [singleton x] returns the one-element set containing only [x]. *)
let remove = remove
        (* [remove x s] returns a set containing all elements of [s],
           except [x]. If [x] was not in [s], [s] is returned unchanged. *)
let union = union
let inter = inter
let diff = diff
        (* Union, intersection and set difference. *)
let compare = compare
        (* Total ordering between sets. Can be used as the ordering function
           for doing sets of sets. *)
let equal = equal
        (* [equal s1 s2] tests whether the sets [s1] and [s2] are
           equal, that is, contain equal elements. *)
let subset = subset
        (* [subset s1 s2] tests whether the set [s1] is a subset of
           the set [s2]. *)
let iter = iter
        (* [iter f s] applies [f] in turn to all elements of [s].
           The order in which the elements of [s] are presented to [f]
           is unspecified. *)
let fold = fold
        (* [fold f s a] computes [(f xN ... (f x2 (f x1 a))...)],
           where [x1 ... xN] are the elements of [s].
           The order in which elements of [s] are presented to [f] is
           unspecified. *)
let cardinal = cardinal
        (* Return the number of elements of a set. *)
let elements = elements
        (* Return the list of all elements of the given set.
           The returned list is sorted in increasing order with respect
           to the ordering [Ord.compare], where [Ord] is the argument
           given to [Set.Make]. *)
let min_elt = min_elt
        (* Return the smallest element of the given set
           (with respect to the [Ord.compare] ordering), or raise
           [Not_found] if the set is empty. *)
let max_elt = max_elt
        (* Same as [min_elt], but returns the largest element of the
           given set. *)
let choose = choose
        (* Return one element of the given set, or raise [Not_found] if
           the set is empty. Which element is chosen is unspecified,
           but equal elements will be chosen for equal sets. *)
;;

