{
open Parser;;
exception Eof;;
exception Illegal;;

let initial_string_buffer = String.create 256
let string_buff = ref initial_string_buffer
let string_index = ref 0

let reset_string_buffer () =
  string_buff := initial_string_buffer;
  string_index := 0

let store_string_char c =
  if !string_index >= String.length (!string_buff) then (
    let new_buff = String.create (String.length (!string_buff) *2) in
    String.blit (!string_buff) 0 new_buff 0 (String.length (!string_buff));
    string_buff := new_buff);
  String.unsafe_set (!string_buff) (!string_index) c;
  incr string_index

let get_stored_string () =
  let s = String.sub (!string_buff) 0 (!string_index) in
  string_buff := initial_string_buffer;
  s

let string_start_pos = ref 0
} 
rule token = parse
  [' ' '\t' '\n']
                   {token lexbuf}
| "("              { LPAREN }
| ")"              { RPAREN }
| ['0'-'9']+       { INT(int_of_string(Lexing.lexeme lexbuf)) }
| "IF"             { IF }
| "ELSEIF"         { ELSEIF }
| "DECISION"       { DECISION }
| "CASE"           { CASE }
| "ARM"            { ARM }
| "EQUALS"         { EQUALS }
| "AND"            { AND }
| "OR"             { OR }
| "VAR"            { VAR }
| "_"              { WILDCARD }
| "\""
    { reset_string_buffer ();
      let string_start = Lexing.lexeme_start lexbuf in
      string_start_pos := string_start;
      string lexbuf;
      lexbuf.Lexing.lex_start_pos <-
	string_start - lexbuf.Lexing.lex_abs_pos;
      STRING (get_stored_string ())
    }    
| eof              { raise Eof }
| _                { raise Illegal }
    
and string = parse
    '"'
    {()}
| _
    { store_string_char(Lexing.lexeme_char lexbuf 0);
      string lexbuf }
