%{
open Syntax;;
%}
%token <string> STRING
%token <int> INT
%token LPAREN RPAREN IF ELSEIF DECISION CASE ARM EQUALS AND OR VAR WILDCARD
  
%start character
%type <Syntax.character> character
%%  

character:
  LPAREN rule_list                     { $2 }
;

rule_list:
  rule rule_list                       { $1 :: $2 }
| RPAREN                               { [] }
;

rule:
  LPAREN inside_rule RPAREN            { Rule ((fst $2), (snd $2)) }
;  

inside_rule:
  state_number stmt                    { ($1::[], $2) }
| state_number inside_rule             { ($1::(fst $2), (snd $2)) }
;    

stmt:
  LPAREN IF condition stmt LPAREN elseif_list stmt RPAREN  { If ($3, $4, $6, $7) }
| LPAREN DECISION new_state utterance RPAREN               { Decision ($3, $4) }
| LPAREN CASE variable LPAREN arm_list stmt RPAREN         { Case ($3, $5, $6) }
;    
  
elseif_list:
  RPAREN                               { [] }
| elseif elseif_list                   { $1 :: $2 }
;
  
elseif:
  LPAREN ELSEIF condition stmt RPAREN  { Elseif ($3, $4) }
;

arm_list:
  RPAREN                               { [] }
| arm arm_list                         { $1 :: $2 }
;

arm:
  LPAREN ARM LPAREN int_list stmt RPAREN         { Arm($4, $5) }
;

int_list:
  RPAREN                               { [] }
| INT int_list                         { $1::$2 }
;

condition:
  LPAREN EQUALS variable INT RPAREN    { Equals($3, $4) }
| LPAREN AND condition_list            { And $3 }
| LPAREN OR condition_list             { Or $3 }
;

condition_list:
  RPAREN                               { [] }
| condition condition_list             { $1::$2 }
;

variable:
  LPAREN VAR STRING RPAREN             { $3 }
;

state_number:
  INT                                  { $1 }
;

new_state:
  INT                                  { Some $1 }
| WILDCARD                             { None }
;

utterance:
  STRING                               { $1 }
;
