open Syntax
open Lexer

let parse_phrase aparser alexer lexbuf =
  let orig = lexbuf.Lexing.lex_abs_pos + lexbuf.Lexing.lex_curr_pos in
  let orig = if orig > 0 then orig+1 else orig in
  try
    let phrase = aparser alexer lexbuf in Parsing.clear_parser(); phrase
  with Parsing.Parse_error ->
    let pos1 = Lexing.lexeme_start lexbuf - orig in
    let pos2 = Lexing.lexeme_end lexbuf - orig in
    failwith
      ("Characters "^(string_of_int pos1)^"-"^
       (string_of_int pos2)^":Syntax error")
  | Lexer.Illegal ->
      let pos1 = Lexing.lexeme_start lexbuf - orig in
      let pos2 = Lexing.lexeme_end lexbuf - orig in
      failwith
        ("Characters "^(string_of_int pos1)^"-"^
         (string_of_int pos2)^":Lexical error "^
         (Lexing.lexeme lexbuf))
	;;

let main () =
  let files = ref [] in
  Arg.parse
    ["-O", Arg.Set Enum.opt, "optimize" ;
     "-v",  Arg.Unit (fun () -> Enum.verbose := 1), "be verbose" ;
     "-V",  Arg.Int  (fun i  -> Enum.verbose := i), "be verbose once in i times" ]
    (fun name -> files := name :: !files)
    "usage: zyva [-O|-v|-V i]* name [name]" ;
    match !files with
    | [file] ->
      let lexbuf = Lexing.from_channel (open_in file) in
      let d = parse_phrase Parser.character Lexer.token lexbuf in
      Enum.count d
    | [file2 ; file1] ->
      let lexbuf = Lexing.from_channel (open_in file1) in
      let d1 = parse_phrase Parser.character Lexer.token lexbuf in      
      let lexbuf = Lexing.from_channel (open_in file2) in
      let d2 = parse_phrase Parser.character Lexer.token lexbuf in
      Enum.equiv d1 d2
    | _ -> exit 2

;;

Printexc.catch main ();;
